#include <lz2a.h>

#include <math.h>

#define DCTSZ	8
#define DCTSZ2	(DCTSZ*DCTSZ)

static float dct_cv[DCTSZ2];
static float dct_c[DCTSZ2];

static int dct_ci[DCTSZ2];
static int dct_cvi[DCTSZ2];
static int dct_cvi2[DCTSZ2*DCTSZ2];
static int dct_icvi2[DCTSZ2*DCTSZ2];

//static int qt[DCTSZ2];
static int sorttab[DCTSZ2];
static int rsorttab[DCTSZ2];
static int clc=0;

short quant_lblk[DCTSZ2];
short dequant_lblk[DCTSZ2];
int xf_min, xf_max;

void DCT_MakeSortTab()
{
	int i, j, k, l;

	i=0;
	j=0;
	l=1;
	for(k=0; k<DCTSZ2; k++)
	{
		sorttab[i*DCTSZ+j]=k;
		rsorttab[k]=i*DCTSZ+j;
		if(l)
		{
			if(i==0)
			{
				j++;
				l=0;
			}else if(j==(DCTSZ-1))
			{
				i++;
				l=0;
			}else
			{
				i--;
				j++;
			}
		}else
		{
			if(i==(DCTSZ-1))
			{
				j++;
				l=1;
			}else if(j==0)
			{
				i++;
				l=1;
			}else
			{
				i++;
				j--;
			}
		}
	}
}

void DCT_MakeQuantTab(int *tab, float q, float q1, float q2)
{
	float f, f2, f3;
	int i, j, k;
//	float q, q1, q2, q3;
	float u, v;

//	q=1;
//	q1=3.0;
//	q2=-0.33;

//	q1=0;
//	q2=0;

	for(i=0; i<DCTSZ; i++)
	{
		for(j=0; j<DCTSZ; j++)
		{
			u=i;
			v=j;

			f=q1*u+q1*v;
			f2=q2*u*u+q2*v*v;

			tab[i*DCTSZ+j]=q*(f+f2+1);

			if(j)printf(" ");
			printf("%4d", tab[i*DCTSZ+j]);
		}
		printf("\n");
	}
	printf("\n");
}

void DCT_MakeQuantTabInput(short *in, int cnt, int *tab, float q)
{
	double deltas[DCTSZ2];
	double f, g, te;
	int i, j;

	for(j=0; j<DCTSZ2; j++)
		deltas[j]=0;

	for(i=0; i<cnt; i++)
		for(j=0; j<DCTSZ2; j++)
			deltas[j]+=fabs(in[i*DCTSZ2+j]);
	for(j=0; j<DCTSZ2; j++)
		deltas[j]/=cnt;

	te=0;
	for(j=0; j<DCTSZ2; j++)
		te+=deltas[j];

#if 0
	for(i=0; i<DCTSZ; i++)
	{
		for(j=0; j<DCTSZ; j++)
		{
			printf("%d\t", (int)deltas[i*DCTSZ+j]);
		}
		printf("\n");
	}
	printf("e %d\n", (int)te);
	printf("\n");
#endif

	for(i=0; i<DCTSZ; i++)
	{
		for(j=0; j<DCTSZ; j++)
		{
			f=deltas[i*DCTSZ+j];

//			f=f/te;
//			f=(1.0-q)/f;

			f=f/te;
			if(f==0)f=0.00001;
			g=(1.0-q);
			f=10*g*g*g/f;

			if(f<1)f=1;
			if(f>32767)f=32767;

//			printf("%d\t", (int)f);
			tab[i*DCTSZ+j]=f;
		}
//		printf("\n");
	}

//	printf("\n");
}

void DCT_Init()
{
	float c1[DCTSZ], f, f2, f3;
	int i, j, k, l, t;
	float u, v;
	float q1, q2, q3;

	if(clc)return;
	clc=1;	

	for(i=0; i<DCTSZ; i++)
		c1[i]=sqrt(2.0/DCTSZ);
	c1[0]=sqrt(1.0/DCTSZ);

//	for(i=0; i<DCTSZ; i++)for(j=0; j<DCTSZ; j++)
//		c[i*DCTSZ+j]=c1[i]*c1[j];
//	for(i=0; i<DCTSZ; i++)for(j=0; j<DCTSZ; j++)
//		cv[i*DCTSZ+j]=cos((M_PI*i)/(2*DCTSZ)*(2*j+1));

	for(i=0; i<DCTSZ; i++)for(j=0; j<DCTSZ; j++)
	{
		dct_c[i*DCTSZ+j]=c1[i]*c1[j];
		dct_ci[i*DCTSZ+j]=c1[i]*c1[j]*256;
	}

	for(i=0; i<DCTSZ; i++)for(j=0; j<DCTSZ; j++)
	{
		f=cos((M_PI*i)/(2*DCTSZ)*(2*j+1));
		dct_cv[i*DCTSZ+j]=f;
		dct_cvi[i*DCTSZ+j]=f*256;
	}

	t=0;
	for(i=0; i<DCTSZ; i++)for(j=0; j<DCTSZ; j++)
	{
		for(k=0; k<DCTSZ; k++)
			for(l=0; l<DCTSZ; l++)
		{
			f=cos((M_PI*i)/(2*DCTSZ)*(2*k+1))*
				cos((M_PI*j)/(2*DCTSZ)*(2*l+1));
			dct_cvi2[t]=f*256;

			f=dct_c[(k*DCTSZ)+l]*
				cos((M_PI*k)/(2*DCTSZ)*(2*i+1))*
				cos((M_PI*l)/(2*DCTSZ)*(2*j+1));
			dct_icvi2[t]=f*65536;

			t++;
		}
	}

//	for(i=0; i<DCTSZ; i++)
//		c1[i]=DCTSZ*((i+1)/2.0);

//	k=1;
//	q1=0.25;
//	q2=0.001;
//	q3=0.0000001;


//	DCT_MakeQuantTab(qt);
//	DCT_MakeSortTab();
}

#if 0
void DCT_Trans(byte *iblk, short *oblk)
{
	int i, j, k, l;
	int s;

	k=0;
	for(i=0; i<DCTSZ2; i++)
	{
		s=0;
		for(j=0; j<DCTSZ2; j++)
			s+=iblk[j]*dct_cvi2[k++];
		s*=dct_ci[i];
		oblk[i]=s>>16;
	}
}

void IDCT_Trans(short *iblk, byte *oblk)
{
	int i, j, k, l;
	int s;

	k=0;
	for(i=0; i<DCTSZ2; i++)
	{
		s=0;
		for(j=0; j<DCTSZ2; j++)
			s+=iblk[j]*dct_icvi2[k++];
		if(s<0)s=0;
		if(s>((1<<24)-1))s=((1<<24)-1);
		oblk[i]=s>>16;
	}
}
#endif

#if 1
void DCT_Trans(byte *iblk, short *oblk)
{
	int i, j, k, l;
	float s;

	for(i=0; i<DCTSZ; i++)for(j=0; j<DCTSZ; j++)
	{
		s=0;
		for(k=0; k<DCTSZ; k++)for(l=0; l<DCTSZ; l++)
		{
			s+=iblk[(k*DCTSZ)+l]*dct_cv[(i*DCTSZ)+k]*dct_cv[(j*DCTSZ)+l];
		}
		s*=dct_c[(i*DCTSZ)+j];

		xf_min=(s<xf_min)?s:xf_min;
		xf_max=(s>xf_max)?s:xf_max;

		oblk[(i*DCTSZ)+j]=s;
	}
}

void IDCT_Trans(short *iblk, byte *oblk)
{
	int i, j, k, l;
	float s;

	for(i=0; i<DCTSZ; i++)for(j=0; j<DCTSZ; j++)
	{
		s=0;
		for(k=0; k<DCTSZ; k++)for(l=0; l<DCTSZ; l++)
		{
			s+=dct_c[(k*DCTSZ)+l]*iblk[(k*DCTSZ)+l]*
				dct_cv[(k*DCTSZ)+i]*dct_cv[(l*DCTSZ)+j];
		}
		if(s<0)s=0;
		if(s>255)s=255;

		oblk[(i*DCTSZ)+j]=s;
	}
}
#endif

int DCT_GetImgBlk(byte *blk, int xo, int yo, byte *img, int xs, int ys)
{
	int i, j, k;

	for(i=0; i<DCTSZ; i++)for(j=0; j<DCTSZ; j++)
	{
		k=img[(i+yo)*xs+(j+xo)];
		blk[i*DCTSZ+j]=k;
	}
}

int DCT_PutImgBlk(byte *blk, int xo, int yo, byte *img, int xs, int ys)
{
	int i, j, k;

	for(i=0; i<DCTSZ; i++)for(j=0; j<DCTSZ; j++)
	{
		k=blk[i*DCTSZ+j];
		img[(i+yo)*xs+(j+xo)]=k;
	}
}

static int dct_m;

int DCT_QuantizeBlock(short *iblk, short *oblk, int *tab)
{
	int i, j, k, l;

	for(i=0; i<DCTSZ2; i++)
	{
		k=iblk[i]/tab[i];

		if(!i)
		{
			l=k;
			k-=dct_m;
			dct_m=l;
		}

		oblk[i]=k;
	}
}

int IDCT_QuantizeBlock(short *iblk, short *oblk, int *tab)
{
	int i, j, k, l;

	for(i=0; i<DCTSZ2; i++)
	{
//		j=sorttab[i];
//		j=i;
//		k=iblk[j]*qt[i];
		k=iblk[i]*tab[i];

		if(!i)
		{
			k+=dct_m;
			dct_m=k;
		}

		if(k)
		{
			j=tab[i];
			if(!j)j=1;
			k+=rand()%j;
		}

		oblk[i]=k;
	}
}

int DCT_QuantizeBlocks(short *blks, int cnt, int *tab)
{
	int i;

	dct_m=0;
	for(i=0; i<cnt; i++)
	{
		DCT_QuantizeBlock(blks+i*DCTSZ2, blks+i*DCTSZ2, tab);
	}
}

int DCT_FilterImage(byte *ibuf, short *obuf, int xs, int ys)
{
	int i, j, k, l;
	byte blk[DCTSZ2];
	short tblk[DCTSZ2], tblk2[DCTSZ2];

//	DCT_Init();
//	xf_min=99999;
//	xf_max=-99999;

	dct_m=0;

	k=0;
	for(i=0; i<(ys/DCTSZ); i++)
		for(j=0; j<(xs/DCTSZ); j++)
	{
		DCT_GetImgBlk(blk, j*DCTSZ, i*DCTSZ, ibuf, xs, ys);

//		DCT_Trans(blk, tblk);

		DCT_Trans(blk, obuf+k*64);
		k++;

//		DCT_QuantizeBlock(tblk, tblk2, tab);

//		for(l=0; l<DCTSZ2; l++)
//			obuf[k*64+l]=tblk2[l];
//		k++;
	}

//	printf("dct %d %d\n", xf_min, xf_max);
}

int IDCT_FilterImage(short *ibuf, byte *obuf, int xs, int ys, int *tab)
{
	int i, j, k, l;
	byte blk[DCTSZ2];
	short tblk[DCTSZ2], tblk2[DCTSZ2];

//	DCT_Init();
	dct_m=0;

	k=0;
	for(i=0; i<(ys/DCTSZ); i++)
		for(j=0; j<(xs/DCTSZ); j++)
	{
//		for(k=0; k<DCTSZ; k++)
//			for(l=0; l<DCTSZ; l++)
//				tblk[k*8+l]=ibuf[((i*8+k)*xs)+(j*8+l)];
		for(l=0; l<DCTSZ2; l++)
			tblk[l]=ibuf[k*64+l];
		k++;

		IDCT_QuantizeBlock(tblk, tblk2, tab);
		IDCT_Trans(tblk2, blk);

//		IDCT_Trans(ibuf+k*64, blk);
//		k++;

		DCT_PutImgBlk(blk, j*DCTSZ, i*DCTSZ, obuf, xs, ys);
	}
}
