/*
encoder/decoder for DCT/MDCT encoded blocks

Context:
low 4: bit number
next 6: dct index
high 6: previous bits for value
*/

unsigned char *dctarith_model;	//pairs of counts, DCT model
unsigned char *dctarith_amodel;	//pairs of counts, alpha model
int dctarith_ctx;	//context, used for alpha

int DCTArith_EncodeSymbol(int v, int n)
{
	int i, j, k, c, l;

	l=0;
	i=12;
	while(i--)
	{
		j=(v>>i)&1;

//		c=((l&4095)<<10)+(n<<4)+i;
		c=((l&63)<<10)+(n<<4)+i;
//		c=(n<<4)+i;
//		c=dctarith_ctx;

		k=WArith2_Predict(dctarith_model, c);
		WArith2_Update(dctarith_model, c, j);

		WHUFF_OutputBit(j, k);

//		dctarith_ctx=((dctarith_ctx<<1)|j)&((1<<20)-1);
		l=(l<<1)|j;
	}
}

int DCTArith_DecodeSymbol(int n)
{
	int i, j, k, l, c;

	l=0;
	i=12;
	while(i--)
	{
		c=((l&63)<<10)+(n<<4)+i;
		j=WHUFF_InputModelBit(dctarith_model, c);
		l=(l<<1)|j;
	}
	if(l&2048)l|=(-1<<12);

	return(l);
}

int DCTArith_EncodeBlock(short *blk, int xs)
{
	int i, j;

	for(i=0; i<8; i++)
		for(j=0; j<8; j++)
	{
//		DCTArith_EncodeSymbol(blk[i*xs+j]-(int)mdct_min, i*8+j);
//		DCTArith_EncodeSymbol(blk[i*xs+j], i*8+j);
		DCTArith_EncodeSymbol(blk[i*xs+j], i+j);
	}
}

int DCTArith_DecodeBlock(short *blk, int xs)
{
	int i, j;

	for(i=0; i<8; i++)
		for(j=0; j<8; j++)
//			blk[i*xs+j]=(short)DCTArith_DecodeSymbol(i*8+j);
			blk[i*xs+j]=(short)DCTArith_DecodeSymbol(i+j);
}

int DCTArith_EncodeBlock2(short *blk)
{
	int i, j;

	for(i=0; i<8; i++)
		for(j=0; j<8; j++)
	{
		DCTArith_EncodeSymbol((short)blk[i*8+j], i*8+j);
//		DCTArith_EncodeSymbol((short)blk[i*8+j], i+j);
	}
}

int DCTArith_DecodeBlock2(short *blk)
{
	int i, j;

	for(i=0; i<8; i++)
		for(j=0; j<8; j++)
			blk[i*8+j]=(short)DCTArith_DecodeSymbol(i*8+j);
//			blk[i*8+j]=(short)DCTArith_DecodeSymbol(i+j);
}

int DCTArith_SetupContexts()
{
	int i, j, k;


	for(i=0; i<256; i++)
		for(j=0; j<256; j++)
	{
		k=((i+1)<<16)/(i+j+2);
		warith2_divtab[i+(j<<8)]=k;
		warith2_divtab2[i+(j<<8)]=k<<16;
	}

	j=1<<22;
	if(!dctarith_model)dctarith_model=malloc(j*2*sizeof(unsigned char));
	for(i=0; i<j*2; i++)dctarith_model[i]=0;

	j=1<<10;
	if(!dctarith_amodel)dctarith_amodel=malloc(j*2*sizeof(unsigned char));
	for(i=0; i<j*2; i++)dctarith_amodel[i]=0;

	dctarith_ctx=0;
}

int DCTArith_EncodeImage(short *in, byte *out, int xs, int ys)
{
	int i, j;

	WHUFF_SetupEncode(out);
	DCTArith_SetupContexts();

	for(i=0; i<(ys/8); i++)
		for(j=0; j<(xs/8); j++)
	{
		DCTArith_EncodeBlock(in+(i*8*xs)+(j*8), xs);
	}

	WHUFF_FlushWBits();
	i=huff_winbuf-out;

	return(i);
}

int DCTArith_SetupEncode(byte *out)
{
	WHUFF_SetupEncode(out);
	DCTArith_SetupContexts();
}

int DCTArith_FinishEncode(byte *out)
{
	int i;

	WHUFF_FlushWBits();
	i=huff_winbuf-out;
	return(i);
}

int DCTArith_EncodeBlocks(short *in, int cnt)
{
	int i, j;

	for(i=0; i<cnt; i++)
		DCTArith_EncodeBlock2(in+(i*64));

}

int DCTArith_SetupDecode(byte *in)
{
	WHUFF_SetupDecode(in);
	DCTArith_SetupContexts();
}

int DCTArith_FinishDecode(byte *in)
{
	int i;

	i=huff_winbuf-in;
	return(i);
}

int DCTArith_DecodeBlocks(short *out, int cnt)
{
	int i, j;

	for(i=0; i<cnt; i++)
		DCTArith_DecodeBlock2(out+(i*64));

	return(i);
}



int DCTArith_EncodeAlphaSymbol(int v)
{
	int i, j, k, c, l;

	l=0;
	i=4;
	while(i--)
	{
		j=(v>>i)&1;

		c=(dctarith_ctx<<2)+i;
		k=WArith2_Predict(dctarith_amodel, c);
		WArith2_Update(dctarith_amodel, c, j);
		WHUFF_OutputBit(j, k);
		dctarith_ctx=((dctarith_ctx<<1)|j)&((1<<8)-1);
	}
}

int DCTArith_DecodeAlphaSymbol()
{
	int i, j, k, l, c;

	l=0;
	i=4;
	while(i--)
	{
		c=(dctarith_ctx<<2)+i;
		j=WHUFF_InputModelBit(dctarith_amodel, c);
		dctarith_ctx=((dctarith_ctx<<1)|j)&((1<<8)-1);
		l=(l<<1)|j;
	}
	if(l&8)l|=(-1<<12);

	return(l);
}

int DCTArith_EncodeAlpha(byte *in, int xs, int ys)
{
	int i, j, t;

	for(i=0; i<ys; i++)
		for(j=0; j<xs; j++)
	{
		t=0;
		if(i && j)
			t=(in[i*xs+(j-1)]+
				in[(i-1)*xs+j]+in[(i-1)*xs+(j-1)])/3;
		t=(in[i*xs+j]/17)-(t/17);
		DCTArith_EncodeAlphaSymbol(t);
	}

	return(i);
}

int DCTArith_DecodeAlpha(byte *out, int xs, int ys)
{
	int i, j, t, v;

	for(i=0; i<ys; i++)
		for(j=0; j<xs; j++)
	{
		t=0;
		if(i && j)
			t=(out[i*xs+(j-1)]+
				out[(i-1)*xs+j]+out[(i-1)*xs+(j-1)])/3;
		v=DCTArith_DecodeAlphaSymbol();
		out[i*xs+j]=(v*17)+t;
	}

	return(i);
}
