/*
DCT and Arithmatic coder based image format.

 */

#include <lz2a.h>
#include <time.h>

#include <core_all.c>
#include <mdct.c>
#include <dct.c>
#include <dctarith.c>
#include <tga.c>

#define MAKE_TAG(a, b, c, d)	(a+(b<<8)+(c<<16)+(d<<24))

#define TAG_PDTX	MAKE_TAG('P', 'D', 'T', 'X')
#define TAG_CHNL	MAKE_TAG('C', 'H', 'N', 'L')
#define TAG_IMG 	MAKE_TAG('I', 'M', 'G', ' ')
#define TAG_END		MAKE_TAG('E', 'N', 'D', ' ')

#define TAG_enc		MAKE_TAG('e', 'n', 'c', ' ')
#define TAG_clrs	MAKE_TAG('c', 'l', 'r', 's')
#define TAG_size	MAKE_TAG('s', 'i', 'z', 'e')
#define TAG_qtab	MAKE_TAG('q', 't', 'a', 'b')
#define TAG_qtuv	MAKE_TAG('q', 't', 'u', 'v')
#define TAG_data	MAKE_TAG('d', 'a', 't', 'a')
#define TAG_flag	MAKE_TAG('f', 'l', 'a', 'g')
#define TAG_dAta	MAKE_TAG('d', 'A', 't', 'a')

#define TAG_DCTA	MAKE_TAG('D', 'C', 'T', 'A')
#define TAG_YUVA	MAKE_TAG('Y', 'U', 'V', 'A')
#define TAG_YUV		MAKE_TAG('Y', 'U', 'V', ' ')
#define TAG_YA		MAKE_TAG('Y', 'A', ' ', ' ')
#define TAG_Y		MAKE_TAG('Y', ' ', ' ', ' ')

int ReadInt16(FILE *fd)
{
	int i;

	i=fgetc(fd);
	i+=fgetc(fd)<<8;
	return(i);
}

int WriteInt16(FILE *fd, int v)
{
	int i;

	fputc(v&0xFF, fd);
	fputc((v>>8)&0xFF, fd);
}

int ReadInt32(FILE *fd)
{
	int i;

	i=fgetc(fd);
	i+=fgetc(fd)<<8;
	i+=fgetc(fd)<<16;
	i+=fgetc(fd)<<24;
	return(i);
}

int WriteInt32(FILE *fd, int v)
{
	int i;

	fputc(v&0xFF, fd);
	fputc((v>>8)&0xFF, fd);
	fputc((v>>16)&0xFF, fd);
	fputc((v>>24)&0xFF, fd);
}

int main_rgb2yuv(int r, int g, int b, int *yp, int *up, int *vp)
{
	int y, u, v;

	y= (0.257*r)+(0.504*g)+(0.098*b)+16;
	u=-(0.148*r)-(0.291*g)+(0.439*b)+128;
	v= (0.439*r)-(0.368*g)-(0.071*b)+128;

//	y = 0.299*r + 0.587*g + 0.114*b + 16;
//	u =-0.169*r - 0.331*g + 0.500*b + 128;
//	v = 0.500*r - 0.419*g - 0.081*b + 128;


//	v=r;
//	y=g;
//	u=b;

//	y=0.299*r + 0.587*g + 0.114*b;
//	u=(b-y)*0.565+128;
//	v=(r-y)*0.713+128;

#if 0
	y=(g*0.5)+(r*0.23)+(b*0.23);
//	y=(g*0.33)+(r*0.33)+(b*0.33);
	u=(b-y)*0.5+128;
	v=(r-y)*0.5+128;
#endif

	y=(y<16)?16:((y>235)?235:y);
	u=(u<16)?16:((u>239)?239:u);
	v=(v<16)?16:((v>239)?239:v);

	y=(y<0)?0:((y>255)?255:y);
	u=(u<0)?0:((u>255)?255:u);
	v=(v<0)?0:((v>255)?255:v);

	*yp=y;
	*up=u;
	*vp=v;
}

int main_yuv2rgb(int y, int u, int v, int *rp, int *gp, int *bp)
{
	int r, g, b;

//	r=v;
//	g=y;
//	b=u;

	b=1.164*(y-16)+2.018*(u-128);
	g=1.164*(y-16)-0.813*(v-128)-0.391*(u-128);
	r=1.164*(y-16)+1.596*(v-128);
//	r*=1.1;
//	b*=1.1;

//	r = (y-16) + 1.403*(v-128);
//	g = (y-16) - 0.344*(u-128) - 0.714*(v-128);
//	b = (y-16) + 1.770*(u-128);

//	y=(g*0.5)+(r*0.25)+(b*0.25);
//	u=(b-y)*0.5+128;
//	v=(r-y)*0.5+128;

#if 0
	r=y+(v-128)*2;
	b=y+(u-128)*2;
	g=(y-r*0.25-g*0.25)*2;
//	g=(y-r*0.33-g*0.33)*2.5;
#endif

	r=(r<0)?0:((r>255)?255:r);
	g=(g<0)?0:((g>255)?255:g);
	b=(b<0)?0:((b>255)?255:b);

	*rp=r;
	*gp=g;
	*bp=b;
}

//downsample, 2 sets of sizes due to possible padding issues
//assumes (xs2>=(xs1>>1)) && (ys2>=(ys1>>1))
void main_downsample(byte *buf, int xs1, int ys1, int xs2, int ys2)
{
	int i, j, k, l;

	for(i=0; i<(ys1>>1); i++)
		for(j=0; j<(xs1>>1); j++)
	{
		l=(i<<1)*xs1+(j<<1);
		k=buf[l]+buf[l+xs1]+buf[l+1]+buf[l+xs1+1];
		k>>=2;

		buf[i*xs2+j]=k;
	}
}

//upsamples in place, similar assumptions hold to that of the previous
//will work backwards
void main_upsample(byte *buf, int xs1, int ys1, int xs2, int ys2)
{
	int i, j, k, l;

	//work backwards to avoid trashing input
	for(i=(ys1>>1)-1; i>=0; i--)
		for(j=(xs1>>1)-1; j>=0; j--)
	{
		k=buf[i*xs2+j];

		l=(i<<1)*xs1+(j<<1);
		buf[l]=k;
		buf[l+xs1]=k;
		buf[l+1]=k;
		buf[l+xs1+1]=k;
	}
}

int main_encode(char *infile, char *outfile, int flag, float q)
{
	FILE *fd;
	byte *ybuf, *ubuf, *vbuf, *abuf;
	byte *ibuf, *obuf, *ibuf2, *obuf2;
	byte *cbuf;
	short *dybuf, *dubuf, *dvbuf;
	int i, j, k, l;
	int xs, ys, xs2, ys2, xs3, ys3;
	int r, g, b, a, y, u, v;
	int clrs;
	int qt[DCTSZ2];
	int qtu[DCTSZ2];
	int qtv[DCTSZ2];
	int qtuv[DCTSZ2];


	if(((flag>>16)&3)==0)clrs=TAG_YUVA;
	if(((flag>>16)&3)==1)clrs=TAG_YA;
	if(((flag>>16)&3)==2)clrs=TAG_YUV;
	if(((flag>>16)&3)==3)clrs=TAG_Y;

	fd=fopen(infile, "rb");
	ibuf=Img_LoadTGA(fd, &xs, &ys);
	fclose(fd);

	xs2=(xs+7)&(~7);
	ys2=(ys+7)&(~7);

	if(flag&1)
	{
		xs3=xs2>>1;
		ys3=ys2>>1;

		xs3=(xs3+7)&(~7);
		ys3=(ys3+7)&(~7);
	}else
	{
		xs3=xs2;
		ys3=ys2;
	}

	ybuf=malloc(xs2*ys2);
	ubuf=malloc(xs2*ys2);
	vbuf=malloc(xs2*ys2);
	abuf=malloc(xs2*ys2);

//	obuf2=malloc(xs2*ys2);

	dybuf=malloc(xs2*ys2*2);
	dubuf=malloc(xs2*ys2*2);
	dvbuf=malloc(xs2*ys2*2);

//	memset(ibuf2, 0, xs2*ys2);
//	memset(obuf2, 0, xs2*ys2);

	memset(dybuf, 0, xs2*ys2*2);
	memset(dubuf, 0, xs2*ys2*2);
	memset(dvbuf, 0, xs2*ys2*2);

	for(i=0; i<ys; i++)
		for(j=0; j<xs; j++)
	{
		r=ibuf[(i*xs+j)*4+0];
		g=ibuf[(i*xs+j)*4+1];
		b=ibuf[(i*xs+j)*4+2];
		a=ibuf[(i*xs+j)*4+3];

		//192: 49.344 96.768 18.816, 164.928
		main_rgb2yuv(r, g, b, &y, &u, &v);

		ybuf[(i*xs2)+j]=y;
		ubuf[(i*xs2)+j]=u;
		vbuf[(i*xs2)+j]=v;
		abuf[(i*xs2)+j]=a;
	}

	if(flag&1)
	{
		main_downsample(ubuf, xs2, ys2, xs3, ys3);
		main_downsample(vbuf, xs2, ys2, xs3, ys3);
	}

	cbuf=malloc(xs2*ys2*8);

	j=clock();

	DCT_Init();

#if 0
	DCT_MakeQuantTab(qt, 1, 3, -0.33);
//	DCT_MakeQuantTab(qt, 1, 6, -0.66);

//	for(i=0; i<8; i++)for(j=0; j<8; j++)
//		if(i>=4 && j>=4)qt[i*8+j]=32767;

//	DCT_MakeQuantTab(qtuv, 1, 3, 0);
	DCT_MakeQuantTab(qtuv, 1, 10, 25);

	for(i=0; i<8; i++)for(j=0; j<8; j++)
		if(i>=4 && j>=4)qtuv[i*8+j]=32767;
#endif

	DCTArith_SetupEncode(cbuf);

	DCT_FilterImage(ybuf, dybuf, xs2, ys2);
	if((clrs==TAG_YUV) || (clrs==TAG_YUVA))
	{
		DCT_FilterImage(ubuf, dubuf, xs3, ys3);
		DCT_FilterImage(vbuf, dvbuf, xs3, ys3);
	}

	DCT_MakeQuantTabInput(dybuf, (xs2/8)*(ys2/8), qt, q);
	if((clrs==TAG_YUV) || (clrs==TAG_YUVA))
	{
		DCT_MakeQuantTabInput(dubuf, (xs3/8)*(ys3/8), qtu, q);
		DCT_MakeQuantTabInput(dubuf, (xs3/8)*(ys3/8), qtv, q);
		for(i=0; i<DCTSZ2; i++)
			qtuv[i]=(qtu[i]+qtv[i])/2;
	}

	DCT_QuantizeBlocks(dybuf, (xs2/8)*(ys2/8), qt);
	if((clrs==TAG_YUV) || (clrs==TAG_YUVA))
	{
		DCT_QuantizeBlocks(dubuf, (xs3/8)*(ys3/8), qtuv);
		DCT_QuantizeBlocks(dvbuf, (xs3/8)*(ys3/8), qtuv);
	}

	DCTArith_EncodeBlocks(dybuf, (xs2/8)*(ys2/8));
	if((clrs==TAG_YUV) || (clrs==TAG_YUVA))
	{
		DCTArith_EncodeBlocks(dubuf, (xs3/8)*(ys3/8));
		DCTArith_EncodeBlocks(dvbuf, (xs3/8)*(ys3/8));
	}

	if((clrs==TAG_YUVA) || (clrs==TAG_YA))
	{
		DCTArith_EncodeAlpha(abuf, xs2, ys2);
	}

	l=DCTArith_FinishEncode(cbuf);

	printf("arith %d %fs\n", l, (clock()-j)/(float)CLOCKS_PER_SEC);

	fd=fopen(outfile, "wb");

	WriteInt32(fd, TAG_PDTX);
	WriteInt32(fd, TAG_IMG);

	WriteInt32(fd, TAG_enc);
	WriteInt32(fd, TAG_DCTA);
	WriteInt32(fd, TAG_clrs);
//	WriteInt32(fd, TAG_YUVA);
	WriteInt32(fd, clrs);

	WriteInt32(fd, TAG_flag);
	WriteInt32(fd, flag);

	WriteInt32(fd, TAG_size);
	WriteInt16(fd, xs);
	WriteInt16(fd, ys);

	WriteInt32(fd, TAG_qtab);
	for(i=0; i<DCTSZ2; i++)
		WriteInt16(fd, qt[i]);

	WriteInt32(fd, TAG_qtuv);
	for(i=0; i<DCTSZ2; i++)
		WriteInt16(fd, qtuv[i]);

	WriteInt32(fd, TAG_data);
	WriteInt32(fd, l);
	fwrite(cbuf, 1, l, fd);

	WriteInt32(fd, TAG_END);
	WriteInt32(fd, TAG_END);
}

int main_decode(char *infile, char *outfile, int flag2)
{
	FILE *fd;
	byte *ybuf, *ubuf, *vbuf, *abuf;
	byte *ibuf, *obuf, *ibuf2, *obuf2;
	byte *cbuf;
	short *dtbuf;
	int i, j, k, xs, ys, xs2, ys2, xs3, ys3;
	int r, g, b, a, y, u, v;
	int flag, clrs;
	int qt[DCTSZ2];
	int qtuv[DCTSZ2];

	fd=fopen(infile, "rb");

	i=ReadInt32(fd);
	if(i!=TAG_PDTX)
	{
		printf("decode: bad file %X\n", i);
		return(-1);
	}

	i=ReadInt32(fd);
	if(i!=TAG_IMG)
	{
		printf("decode: unrecognized layout\n");
		return(-1);
	}

	DCT_Init();

	while(1)
	{
		i=ReadInt32(fd);
		if(i==TAG_END)break;

		if(i==TAG_enc)
		{
			j=ReadInt32(fd);
			if(j!=TAG_DCTA)
			{
				printf("unknown encoding\n");
				return(-1);
			}
		}

		if(i==TAG_clrs)
		{
			j=ReadInt32(fd);
			if(j!=TAG_YUVA && j!=TAG_YUV &&
				j!=TAG_YA && j!=TAG_Y)
			{
				printf("unknown colorspace\n");
				return(-1);
			}
			clrs=j;
		}

		if(i==TAG_size)
		{
			xs=ReadInt16(fd);
			ys=ReadInt16(fd);
		}

		if(i==TAG_flag)
		{
			flag=ReadInt32(fd);
		}

		if(i==TAG_qtab)
		{
			for(j=0; j<DCTSZ2; j++)
			{
				qt[j]=ReadInt16(fd);
				qtuv[j]=qt[j];
			}
		}
		if(i==TAG_qtuv)
		{
			for(j=0; j<DCTSZ2; j++)
				qtuv[j]=ReadInt16(fd);
		}

		if(i==TAG_data)
		{
			j=ReadInt32(fd);
			cbuf=malloc(j);

			fread(cbuf, 1, j, fd);
		}
	}

	xs2=(xs+7)&(~7);
	ys2=(ys+7)&(~7);

	if(flag&1)
	{
		xs3=xs2>>1;
		ys3=ys2>>1;

		xs3=(xs3+7)&(~7);
		ys3=(ys3+7)&(~7);
	}else
	{
		xs3=xs2;
		ys3=ys2;
	}

	ibuf2=malloc(xs2*ys2);
	obuf2=malloc(xs2*ys2);
	dtbuf=malloc(xs2*ys2*4);

	ybuf=malloc(xs2*ys2);
	ubuf=malloc(xs2*ys2);
	vbuf=malloc(xs2*ys2);
	abuf=malloc(xs2*ys2);


	memset(ibuf2, 0, xs2*ys2);
	memset(obuf2, 0, xs2*ys2);
	memset(dtbuf, 0, xs2*ys2*4);

	memset(ybuf, 128, xs2*ys2);
	memset(ubuf, 128, xs2*ys2);
	memset(vbuf, 128, xs2*ys2);
	memset(abuf, 255, xs2*ys2);

	j=clock();

	DCTArith_SetupDecode(cbuf);

	DCTArith_DecodeBlocks(dtbuf, (xs2/8)*(ys2/8));
	IDCT_FilterImage(dtbuf, ybuf, xs2, ys2, qt);

	if((clrs==TAG_YUV) || (clrs==TAG_YUVA))
	{
		DCTArith_DecodeBlocks(dtbuf, (xs3/8)*(ys3/8));
		IDCT_FilterImage(dtbuf, ubuf, xs3, ys3, qtuv);
		DCTArith_DecodeBlocks(dtbuf, (xs3/8)*(ys3/8));
		IDCT_FilterImage(dtbuf, vbuf, xs3, ys3, qtuv);
	}

	if((clrs==TAG_YUVA) || (clrs==TAG_YA))
	{
//		DCTArith_DecodeBlocks(dtbuf, (xs2/8)*(ys2/8));
//		IDCT_FilterImage(dtbuf, abuf, xs2, ys2);
		DCTArith_DecodeAlpha(abuf, xs2, ys2);
	}

	i=DCTArith_FinishDecode(cbuf);
	printf("arith %d %fs\n", i, (clock()-j)/(float)CLOCKS_PER_SEC);

	if((clrs==TAG_YUV) || (clrs==TAG_YUVA) && (flag&1))
	{
		main_upsample(ubuf, xs2, ys2, xs3, ys3);
		main_upsample(vbuf, xs2, ys2, xs3, ys3);
	}

	obuf=malloc(xs*ys*4);
	for(i=0; i<ys; i++)
		for(j=0; j<xs; j++)
	{
		y=ybuf[(i*xs2)+j];
		u=ubuf[(i*xs2)+j];
		v=vbuf[(i*xs2)+j];
		a=abuf[(i*xs2)+j];

		main_yuv2rgb(y, u, v, &r, &g, &b);

		obuf[(i*xs+j)*4+0]=r;
		obuf[(i*xs+j)*4+1]=g;
		obuf[(i*xs+j)*4+2]=b;
		obuf[(i*xs+j)*4+3]=a;
	}

	fd=fopen(outfile, "wb");
	Img_StoreTGA(fd, obuf, xs, ys);
	fclose(fd);
}

int help(int argc, char *argv[])
{
	printf("Usage: %s [options] [infile [outfile]]\n\n", argv[0]);
	printf("Options:\n"
	"	-h	This message\n"
	"	-c	Compress infile to outfile\n"
	"	-d	Decompress infile to outfile\n"
	"	-ds	Downsample UV during compression\n"
	"	-nds	Don't downsample UV during compression\n"
	"	-nuv	Omit UV channels in images\n"
	"	-na	Omit alpha channel in images\n"
	"	-nuva	Omit UV and alpha channels in images\n"
	"	-q <f>	Set quality level as a float between 0 and 1\n"
	"\n"
	"This tool does not allow option globbing.\n"
	"Only TGA and PDTX (compressed output) are supported formats presently.\n"
	);
}

int main(int argc, char *argv[])
{
	FILE *fd;
//	byte *ibuf, *obuf, *ibuf2, *obuf2;
//	int r, g, b, y, u, v;
//	int xs, ys;

	char *infile, *outfile;
	int flag, cd;
	float q;

	int i, j;

	infile=NULL;
	outfile=NULL;
	flag=1;
	cd=0;
	q=0.85;

	i=1;
	while(i<argc)
	{
		if(argv[i][0]=='-')
		{
			if(!strcmp(argv[i], "-h"))
			{
				help(argc, argv);
				return(0);
			}else if(!strcmp(argv[i], "-c"))
			{
				cd=1;
			}else if(!strcmp(argv[i], "-d"))
			{
				cd=2;
			}else if(!strcmp(argv[i], "-ds"))
			{
				flag|=1;
			}else if(!strcmp(argv[i], "-nds"))
			{
				flag&=~1;
			}else if(!strcmp(argv[i], "-nuv"))
			{
				flag|=65536;
			}else if(!strcmp(argv[i], "-na"))
			{
				flag|=131072;
			}else if(!strcmp(argv[i], "-nuva"))
			{
				flag|=65536|131072;
			}else if(!strcmp(argv[i], "-q"))
			{
				q=atof(argv[i+1]);
				if(q<0)
				{
					printf("%s: quality clamp 0\n",
						argv[0]);
					q=0;
				}
				if(q>1)
				{
					printf("%s: quality clamp 1\n",
						argv[0]);
					q=1;
				}

				i+=2;
				continue;
			}else
			{
				printf("%s: unrecognized option '%s'\n",
					argv[0], argv[i]);
				help(argc, argv);
				return(-1);
			}

			i++;
			continue;
		}

		if(!infile)
		{
			infile=argv[i];
		}else if(!outfile)
		{
			outfile=argv[i];
		}else
		{
			printf("%s: too many files\n", argv[0]);
			help(argc, argv);
			return(-1);
		}
		i++;
		continue;
	}

	if(cd==1 && infile && outfile)
	{
		main_encode(infile, outfile, flag, q);
	}else if(cd==2 && infile && outfile)
	{
		main_decode(infile, outfile, flag);
	}else
	{
		printf("%s: problem with mode or files given\n", argv[0]);
		help(argc, argv);
		return(-1);
	}

	return(0);

#if 0
	if(!strcmp(argv[1], "-c"))
		main_encode(argv[2], argv[3], 1, 0.8);
	if(!strcmp(argv[1], "-d"))
		main_decode(argv[2], argv[3], 0);

	if(!strcmp(argv[1], "-cst"))
	{
		fd=fopen(argv[2], "rb");
		ibuf=Img_LoadTGA(fd, &xs, &ys);
		fclose(fd);

		obuf=malloc(xs*ys*4);
		for(i=0; i<ys; i++)
			for(j=0; j<xs; j++)
		{
			r=ibuf[(i*xs+j)*4+0];
			g=ibuf[(i*xs+j)*4+1];
			b=ibuf[(i*xs+j)*4+2];

			main_rgb2yuv(r, g, b, &y, &u, &v);
			main_yuv2rgb(y, u, v, &r, &g, &b);

			obuf[(i*xs+j)*4+0]=r;
			obuf[(i*xs+j)*4+1]=g;
			obuf[(i*xs+j)*4+2]=b;
			obuf[(i*xs+j)*4+3]=255;
		}

		fd=fopen(argv[3], "wb");
		Img_StoreTGA(fd, obuf, xs, ys);
		fclose(fd);
	}
#endif

#if 0
	FILE *fd;
	byte *ibuf, *obuf, *ibuf2, *obuf2;
	byte *cbuf;
	short *dtbuf;
	int i, j, k, xs, ys, xs2, ys2;

	fd=fopen(argv[1], "rb");
	ibuf=Img_LoadTGA(fd, &xs, &ys);
	fclose(fd);

	xs2=(xs+7)&(~7);
	ys2=(ys+7)&(~7);

	ibuf2=malloc(xs2*ys2);
	obuf2=malloc(xs2*ys2);
	dtbuf=malloc(xs2*ys2*4);

	memset(ibuf2, 0, xs2*ys2);
	memset(obuf2, 0, xs2*ys2);
	memset(dtbuf, 0, xs2*ys2*4);

	for(i=0; i<ys; i++)
		for(j=0; j<xs; j++)
	{
//		ibuf2[((i+8)*xs2)+(j+8)]=
		ibuf2[(i*xs2)+j]=
			ibuf[(i*xs+j)*4+0]*0.33+
			ibuf[(i*xs+j)*4+1]*0.42+
			ibuf[(i*xs+j)*4+2]*0.25;
	}

//	MDCT_FilterImage(ibuf2, dtbuf, xs2, ys2);
//	IMDCT_FilterImage(dtbuf, obuf2, xs2, ys2);

	j=clock();
	DCT_FilterImage(ibuf2, dtbuf, xs2, ys2);
	printf("dct %fs\n", (clock()-j)/(float)CLOCKS_PER_SEC);

	cbuf=malloc(xs2*ys2*4);
//	i=DCTArith_EncodeImage(dtbuf, cbuf, xs2, ys2);

	j=clock();
	i=DCTArith_EncodeBlocks(dtbuf, cbuf, (xs2/8)*(ys2/8));
	printf("arith %d %fs\n", i, (clock()-j)/(float)CLOCKS_PER_SEC);

	j=clock();
	i=DCTArith_DecodeBlocks(cbuf, dtbuf, (xs2/8)*(ys2/8));
	printf("arith %d %fs\n", i, (clock()-j)/(float)CLOCKS_PER_SEC);

	j=clock();
	IDCT_FilterImage(dtbuf, obuf2, xs2, ys2);
	printf("idct %fs\n", (clock()-j)/(float)CLOCKS_PER_SEC);

	obuf=malloc(xs*ys*4);
	for(i=0; i<ys; i++)
		for(j=0; j<xs; j++)
	{
//		k=obuf2[((i+8)*xs2)+(j+8)];
		k=obuf2[(i*xs2)+j];
		obuf[(i*xs+j)*4+0]=k;
		obuf[(i*xs+j)*4+1]=k;
		obuf[(i*xs+j)*4+2]=k;
		obuf[(i*xs+j)*4+3]=255;
	}

	fd=fopen(argv[2], "wb");
	Img_StoreTGA(fd, obuf, xs, ys);
	fclose(fd);
#endif
}
