/*
audio-intended mdct
 */

#include <lz2a.h>

#include <math.h>

#define MDCT_BLKSZ	256
#define MDCT_QUANT_BASE	1024
#define MDCT_QUANT_SQRT	0 //4096
#define MDCT_QUANT_LIN	4096
#define MDCT_QUANT_SQ	0

//#define MDCT_USEIDX

//#define MDCT_BLKSZ	64
//#define MDCT_QUANT	2048

//#define MDCT_BLKSZ	1024
//#define MDCT_QUANT	16384

int mdct_quant[MDCT_BLKSZ];
int mdct_min, mdct_max;

int MDCT_Init()
{
	int i, j;
	float q0, q1, q2, q3;
	float f, g;

	q0=MDCT_BLKSZ;
	q1=0;
	q2=0;
	q3=0;

	//approx 40
//	q0=1024;
//	q1=4096;
//	q2=-1024;
//	q3=64;

	//approx 67
//	q0=256;
//	q1=1024;
//	q2=0;
//	q3=0;

//	q1=256;

//	q0=2048;
	q0=4096;

	g=44100/MDCT_BLKSZ;
	for(i=0; i<8; i++)
	{
		j=(1<<i);
//		j=i*(MDCT_BLKSZ/16);
		f=q0 + q1*sqrt(j) + q2*j + q3*j*j;
		printf("%dHz\t%d\n", (int)(j*g), (int)f);
	}
	printf("\n");

	for(i=0; i<MDCT_BLKSZ; i++)
	{
		mdct_quant[i]=q0 + q1*sqrt(i) + q2*i + q3*i*i;
	}
}

int MDCT_BlockN(short *in, int *out, int n)
{
	int i, j, k;
	float f, g, h;

	for(i=0; i<n; i++)
	{
		f=0;

		for(j=0; j<(n*2); j++)
		{
			g=in[j];

//			g*=sin(M_PI*(j+0.5)/(2*n));
			h=sin(M_PI*(j+0.5)/(2*n));
			g*=sin(M_PI*(h*h)/2);

			f+=g*cos(M_PI*(i+0.5)*(j+0.5+n*0.5)/n);
		}
		out[i]=f;

//		out[i]=in[i];

		if(f<mdct_min)mdct_min=f;
		if(f>mdct_max)mdct_max=f;
	}
}

int IMDCT_BlockN(int *in, short *out, int n)
{
	int i, j, k;
	float f, g;

	for(i=0; i<(n*2); i++)
	{
		f=0;

		for(j=0; j<n; j++)
		{
			g=in[j];
			f+=g*cos(M_PI*(j+0.5)*(i+0.5+n*0.5)/n);
		}

//		f*=sin(M_PI*(i+0.5)/(2*n));
		g=sin(M_PI*(i+0.5)/(2*n));
		f*=sin(M_PI*(g*g)/2);

		k=out[i]+(2*f/n);
//		k=out[i]+(f/n);

		out[i]=k;

//		out[i]=in[i]*2;
	}
}

int MDCT_FilterAudio(short *in, int *out, int *idx, int cnt)
{
	int blk[1024];
	int i, j, k, l, li;
	int m, n;
	int bn, bi;
	int min2, max2;
	float e;

	MDCT_Init();

	mdct_min=999999999;
	mdct_max=-999999999;

	min2=999999999;
	max2=-999999999;

//	for(j=0; j<MDCT_BLKSZ; j++)lblk[j]=0;
	e=0;

//	printf("\n");
	for(i=0; i<(cnt/MDCT_BLKSZ); i++)
	{
		printf("blk %d/%d avg e %f\r", i, cnt/MDCT_BLKSZ, e/(i+1));
		fflush(stdout);

		MDCT_BlockN(in+i*MDCT_BLKSZ, blk, MDCT_BLKSZ);

		//approximate entropy
		l=0;
		m=0;
		for(j=0; j<MDCT_BLKSZ; j++)
		{
			k=blk[j];

			if(k<0)k=-k;
			if(k>m)m=k;
			l+=k;
		}
		e+=l;

/*
		l/=MDCT_BLKSZ;
		if(l<1)l=1;
		l=m/l;
		if(l<1)l=1;
		l=32767/l;
*/
		j=l/MDCT_BLKSZ;
		if(j<1)j=1;
		j=m-j;
		if(j<1)j=1;
		l=l/j;

		m/=16384;
		if(l<m)l=m;
		if(l<1)l=1;
		idx[i]=l;

		for(j=0; j<MDCT_BLKSZ; j++)
		{
			k=blk[j];

//			k*=(1.0/MDCT_QUANT);
//			k*=(1.0/(mdct_quant[j]*l));
			k/=l;

			if(k<min2)min2=k;
			if(k>max2)max2=k;
			out[i*MDCT_BLKSZ+j]=k;

//			blk[j]=k;
		}

#ifdef MDCT_USEIDX
		bn=0;
		bi=0;
		for(j=0; j<MDCT_BLKSZ; j++)
		{
			k=blk[j];
			if(k<0)k=-k;
			bn+=k;
		}

		for(j=1; j<((256<i)?256:i); j++)
		{
			l=0;
			for(k=0; k<MDCT_BLKSZ; k++)
			{
				li=out[(i-j)*MDCT_BLKSZ+k]-blk[k];
				if(li<0)li=-li;
				l+=li;
			}

			if(l<bn)
			{
				bn=l;
				bi=j;
			}
		}

		if(bi)for(j=0; j<MDCT_BLKSZ; j++)
		{
			k=blk[j]-out[(i-bi)*MDCT_BLKSZ+j];
//			out[i*MDCT_BLKSZ+j]=k;
			blk[j]=k;
		}
		idx[i]=bi;
#endif
	}
	printf("\n");
	printf("mdct %d %d\n", mdct_min, mdct_max);
	printf("mdct %d %d\n", min2, max2);
}

int IMDCT_FilterAudio(int *in, short *out, int *idx, int cnt)
{
	int blk[1024], lblk[1024];
	int i, j, k;

	for(j=0; j<MDCT_BLKSZ; j++)lblk[j]=0;

	for(i=0; i<(cnt/MDCT_BLKSZ); i++)
	{
		for(j=0; j<MDCT_BLKSZ; j++)
		{
			k=in[i*MDCT_BLKSZ+j];

//			k*=MDCT_QUANT;
//			k*=mdct_quant[j];
//			k*=mdct_quant[j]*idx[i];
			k*=idx[i];

#ifdef MDCT_USEIDX
			k+=in[(i-idx[i])*MDCT_BLKSZ+j];
#endif


//			k+=lblk[j];
//			lblk[j]=k;

			blk[j]=k;
		}

		printf("blk %d/%d\r", i, cnt/MDCT_BLKSZ);
		fflush(stdout);

		IMDCT_BlockN(blk, out+i*MDCT_BLKSZ, MDCT_BLKSZ);
	}
	printf("\n");
}
