/*
encoder/decoder for MDCT encoded (audio) blocks

Context:
low 4: bit number
next 6: dct index
high 6: previous bits for value
*/

unsigned char *mdctarith_model;	//pairs of counts, DCT model

int MDCTArith_EncodeSymbol(int v, int n)
{
	int i, j, k, c, l;

	l=0;
	i=16;
	while(i--)
	{
		j=(v>>i)&1;

		c=((l&63)<<10)+(n<<4)+i;
		k=WArith2_Predict(mdctarith_model, c);
		WArith2_Update(mdctarith_model, c, j);
		WHUFF_OutputBit(j, k);
		l=(l<<1)|j;
	}
}

int MDCTArith_DecodeSymbol(int n)
{
	int i, j, k, l, c;

	l=0;
	i=16;
	while(i--)
	{
		c=((l&63)<<10)+(n<<4)+i;
		j=WHUFF_InputModelBit(mdctarith_model, c);
		l=(l<<1)|j;
	}
	if(l&32768)l|=(-1<<16);

	return(l);
}

int MDCTArith_EncodeBlock(int *blk, int n)
{
	int i, j;

	for(i=0; i<n; i++)
		MDCTArith_EncodeSymbol(blk[i], i);
}

int MDCTArith_DecodeBlock(int *blk, int n)
{
	int i, j;

	for(i=0; i<n; i++)
		blk[i]=MDCTArith_DecodeSymbol(i);
}

int MDCTArith_SetupContexts()
{
	int i, j, k;


	for(i=0; i<256; i++)
		for(j=0; j<256; j++)
	{
		k=((i+1)<<16)/(i+j+2);
		warith2_divtab[i+(j<<8)]=k;
		warith2_divtab2[i+(j<<8)]=k<<16;
	}

	j=1<<22;
	if(!mdctarith_model)mdctarith_model=malloc(j*2*sizeof(unsigned char));
	for(i=0; i<j*2; i++)mdctarith_model[i]=0;
}


int MDCTArith_SetupEncode(byte *out)
{
	WHUFF_SetupEncode(out);
	MDCTArith_SetupContexts();
}

int MDCTArith_FinishEncode(byte *out)
{
	int i;

	WHUFF_FlushWBits();
	i=huff_winbuf-out;
	return(i);
}

int MDCTArith_EncodeBlocks(int *in, int cnt, int sz)
{
	int i, j;

	for(i=0; i<cnt; i++)
		MDCTArith_EncodeBlock(in+(i*sz), sz);

}

int MDCTArith_SetupDecode(byte *in)
{
	WHUFF_SetupDecode(in);
	MDCTArith_SetupContexts();
}

int MDCTArith_FinishDecode(byte *in)
{
	int i;

	i=huff_winbuf-in;
	return(i);
}

int MDCTArith_DecodeBlocks(int *out, int cnt, int sz)
{
	int i, j;

	for(i=0; i<cnt; i++)
		MDCTArith_DecodeBlock(out+(i*sz), sz);

	return(i);
}
